package org.rcisoft.core.auth.config;

import lombok.extern.slf4j.Slf4j;
import okhttp3.Interceptor;
import okhttp3.Request;
import okhttp3.Response;

import org.rcisoft.core.auth.constant.CyAuthConstants;
import org.rcisoft.core.util.CyServiceAuthUtil;
import org.rcisoft.core.util.CyStrUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Component;

import java.io.IOException;

/**
 * Created by lcy on 18/4/9.
 */
@Component
@Slf4j
public class CyOkHttpTokenInterceptor implements Interceptor {
    @Autowired
    @Lazy
    public CyServiceAuthUtil rcServiceAuthUtil;
    @Autowired
    @Lazy
    private CyServiceAuthConfig rcServiceAuthConfig;


    /**
     * 追加token
     * @param chain
     * @return
     * @throws IOException
     */
    @Override
    public Response intercept(Chain chain) throws IOException {

        log.info("url:      " + chain.request().url().toString());

        Request newRequest = null;
        /** 访问 服务 token **/
        if (chain.request().url().toString().contains("/client/token")) {
            newRequest = chain.request()
                    .newBuilder()
                    .build();
        } else {
            newRequest = chain.request()
                    .newBuilder()
                    .header(rcServiceAuthConfig.getTokenHeader(),
                            CyStrUtil.getObjectValue(rcServiceAuthUtil.getClientToken()))
                    .build();
        }
        Response response = chain.proceed(newRequest);
        if (HttpStatus.FORBIDDEN.value() == response.code()) {
            /*客户端token异常*/
            if (response.body().string().contains(String.valueOf(CyAuthConstants.EX_CLIENT_INVALID_CODE))) {
                log.info("Client Token Expire,Retry to request...");
                rcServiceAuthUtil.refreshClientToken();
                newRequest = chain.request()
                        .newBuilder()
                        .header(rcServiceAuthConfig.getTokenHeader(), rcServiceAuthUtil.getClientToken())
                        .build();
                response = chain.proceed(newRequest);
            }
        }
        /*else {
            String json = null;
            ResponseBody responseBody = response.body();
            if (null != responseBody)
                json = responseBody.string();
            CyScResult result = null;
            try {
                result = JSONObject.parseObject(json, CyScResult.class);
                log.info("---" + result);
                if(null != result && CyScResultCode.SUCCESS.code != result.getCode())
                    throw new ServiceException(ResultServiceEnums.FAIL);
            }catch (Exception e){
                log.error(e.getMessage());
            }
        }*/
        return response;
    }
}
